'''
  VC Sampler sound utility - prepareSound v1.0
  by Astro / SYTNHFOX
  part of a "VC Sampler" hybrid hardware/software project
  
  If you found it somewhere else other than my website - 
  go to https://sfcs.neocities.org/module/SFP26/index.html and download the full resource pack
  you WILL need other stuff that comes with it
  
  Don't reuse any of this code for a device that you're going to sell!
  
  Credit me and my website if you use it

'''

from pydub import AudioSegment
import os.path
import os
from sys import platform

'''
some under the hood variables go here
'''
MAXBYTES = 20480 # absolute maximum of sample bytes that's possible to shove up the arduino




'''
backslash bullshit
'''
def preparePathString(arg):
	# prepare the name
	if platform == "linux" or platform == "linux2":
	    # linux
	    arg = arg.replace("\\", "")

	elif platform == "darwin":
	    # OS X
	    arg = arg.replace("\\", "")

	elif platform == "win32":
	    # Windows...
	    arg = arg.replace("\\", "/")

	return arg

'''
checks if a file exists and is .wav, throws errors if not
'''
def prepareWavPath(arg):

	

	arg = preparePathString(arg)

	# check
	if not (os.path.isfile(arg)):
		raise ValueError("bad sample path!")
	fn, fext = os.path.splitext(arg)
	if not (fext == ".wav" or fext == ".WAV"):
		raise ValueError("sample isn't wav!")

	return arg


'''
prepares a .wav for conversion to pasteable text chunk
converts it to a 16 bit mono file with a
'''
def formatWavSound(fname, doubleDensity = False):
	sound = AudioSegment.from_wav(fname)
	sound = sound.apply_gain(-sound.max_dBFS)
	sound = sound.set_channels(1)
	if (doubleDensity):
		sound = sound.set_frame_rate(4000)
	else:
		sound = sound.set_frame_rate(8000)
	sound = sound.set_sample_width(1)
	return sound


'''
returns an array of sample values from an 8-bit sound
samples are 0 TO 255 (not -128 to 127)
'''
def getSoundSamples(aseg):
	samples = aseg.get_array_of_samples().tolist()
	output = []
	for  s in samples:
		output.append(s + 128)
	return output

'''
returns a string that's be pasted to the arduino sketch
sample side means which of the 2 samples it is. False for 1, True for 2.
'''
def getFinalString(samplesList):


	# start type
	output = "const uint8_t sampleFrames"


	# set array size
	output += "["
	arraySize = 0
	for x in samplesList:
		arraySize += len(x)
	output += str(arraySize)
	output += "] PROGMEM = {"


	# fill elements
	counter = 0;
	samplePositions = list()
	samplePos = 0;
	for x in (samplesList):
		output += str(x)[1:-1]
		counter += 1;
		if not (counter == len(samplesList)):
			output += ", "
		samplePositions.append(samplePos)
		samplePos += len(x)
	samplePositions.append(samplePos) # end of the whole thing
	# check for memory overflow!!!
	if (samplePos >= MAXBYTES):
		t = samplePos/1024
		print("[!!!] FILES TOO LARGE: maximum overall size of converted sample set is 20KB (20480B)",
			" the provided sample set's converted size is " + str(samplePos) + "B (approx. ",
			 f"{t:.2f}KB) - please make it smaller!"
			)
		exit()
	output += "};\n"


	# add the sample count variable
	output += "const uint16_t sampleCuts"
	output += "["
	output += str(len(samplePositions))
	output += "] = {"
	for x in range(len(samplePositions)):
		output += str(samplePositions[x])
		if not (x == len(samplePositions)-1):
			output += ", "
	output += "};\n"
	return output


def process(fileList, outDir, dd=False):

	outFile="data.txt"

	samples = list()

	for f in fileList:
		samples .append(getSoundSamples(formatWavSound(f, dd)))


	output = getFinalString(samples)

	target = os.path.join(outDir, outFile)
	if (os.path.isfile(target)):
		os.remove(target)
	f = open(target, "a")
	f.write(output)
	f.close()
	return


'''
target1 = "./test1.wav"
target2 = "./test2.wav"
output = "./test-out.wav"


aseg1 = formatWavSound(target1, False)
aseg2 = formatWavSound(target2, False)

samples1 = getSoundSamples(aseg1)
samples2 = getSoundSamples(aseg2)

print(getFinalString(samples1, samples2))
'''
